@@ -5,11 +5,12 @@ module Cabal
   ) where
 
 #ifdef ENABLE_CABAL
-
+import Stack
 import Control.Exception (IOException, catch)
 import Data.Char (isSpace)
 import Data.List (foldl', nub, sort, find, isPrefixOf, isSuffixOf)
 #if __GLASGOW_HASKELL__ < 709
+import Control.Applicative ((<$>))
 import Data.Monoid (Monoid(..))
 #endif
 import Distribution.Package (PackageIdentifier(..), PackageName)
@@ -40,6 +41,7 @@ import System.IO.Error (ioeGetErrorString)
 import System.Directory (doesFileExist, getDirectoryContents)
 import System.FilePath (takeDirectory, splitFileName, (</>))
 
+
 componentName :: Component -> ComponentName
 componentName =
     foldComponent (const CLibName)
@@ -95,22 +97,33 @@ allComponentsBy pkg_descr f =
                     , benchmarkEnabled bm ]
 #endif
 
+stackifyFlags :: ConfigFlags -> Maybe StackConfig -> ConfigFlags
+stackifyFlags cfg Nothing   = cfg
+stackifyFlags cfg (Just si) = cfg { configDistPref    = toFlag dist
+                                  , configPackageDBs  = pdbs
+                                  }
+    where
+      pdbs                  = [Nothing, Just GlobalPackageDB] ++ pdbs'
+      pdbs'                 = Just . SpecificPackageDB <$> stackDbs si
+      dist                  = stackDist si
 
-getPackageGhcOpts :: FilePath -> IO (Either String [String])
-getPackageGhcOpts path = do
+-- via: https://groups.google.com/d/msg/haskell-stack/8HJ6DHAinU0/J68U6AXTsasJ
+-- cabal configure --package-db=clear --package-db=global --package-db=$(stack path --snapshot-pkg-db) --package-db=$(stack path --local-pkg-db)
+
+getPackageGhcOpts :: FilePath -> Maybe StackConfig -> IO (Either String [String])
+getPackageGhcOpts path mbStack = do
     getPackageGhcOpts' `catch` (\e -> do
         return $ Left $ "Cabal error: " ++ (ioeGetErrorString (e :: IOException)))
   where
     getPackageGhcOpts' :: IO (Either String [String])
     getPackageGhcOpts' = do
         genPkgDescr <- readPackageDescription silent path
-
-        let cfgFlags' = (defaultConfigFlags defaultProgramConfiguration)
+        let cfgFlags'' = (defaultConfigFlags defaultProgramConfiguration)
                             { configDistPref = toFlag $ takeDirectory path </> "dist"
                             -- TODO: figure out how to find out this flag
                             , configUserInstall = toFlag True
                             }
-
+        let cfgFlags'  = stackifyFlags cfgFlags'' mbStack
         let sandboxConfig = takeDirectory path </> "cabal.sandbox.config"
         exists <- doesFileExist sandboxConfig
 
@@ -121,30 +134,31 @@ getPackageGhcOpts path = do
                              return $ cfgFlags'
                                           { configPackageDBs = [Just sandboxPackageDb]
                                           }
-
         localBuildInfo <- configure (genPkgDescr, emptyHookedBuildInfo) cfgFlags
-
         let pkgDescr = localPkgDescr localBuildInfo
         let baseDir = fst . splitFileName $ path
         case getGhcVersion localBuildInfo of
             Nothing -> return $ Left "GHC is not configured"
-            Just ghcVersion -> do
-                let mbLibName = pkgLibName pkgDescr
 
-                let ghcOpts' = foldl' mappend mempty $ map (getComponentGhcOptions localBuildInfo) $ flip allComponentsBy (\c -> c) . localPkgDescr $ localBuildInfo
 #if __GLASGOW_HASKELL__ >= 709
+            Just _  -> do
+                let mbLibName = pkgLibName pkgDescr
+                let ghcOpts' = foldl' mappend mempty $ map (getComponentGhcOptions localBuildInfo) $ flip allComponentsBy (\c -> c) . localPkgDescr $ localBuildInfo
                     -- FIX bug in GhcOptions' `mappend`
                     ghcOpts = ghcOpts' { ghcOptExtra = overNubListR (filter (/= "-Werror")) $ ghcOptExtra ghcOpts'
                                        , ghcOptPackageDBs = sort $ nub (ghcOptPackageDBs ghcOpts')
                                        , ghcOptPackages = overNubListR (filter (\(_, pkgId, _) -> Just (pkgName pkgId) /= mbLibName)) $ (ghcOptPackages ghcOpts')
                                        , ghcOptSourcePath = overNubListR (map (baseDir </>)) (ghcOptSourcePath ghcOpts')
                                        }
-
                 putStrLn "configuring"
                 (ghcInfo,_,_) <- GHC.configure silent Nothing Nothing defaultProgramConfiguration
 
                 return $ Right $ renderGhcOptions ghcInfo ghcOpts
 #else
+            Just ghcVersion -> do
+                let mbLibName = pkgLibName pkgDescr
+                let ghcOpts' = foldl' mappend mempty $ map (getComponentGhcOptions localBuildInfo) $ flip allComponentsBy (\c -> c) . localPkgDescr $ localBuildInfo
+
                     ghcOpts = ghcOpts' { ghcOptExtra = filter (/= "-Werror") $ nub $ ghcOptExtra ghcOpts'
                                        , ghcOptPackages = filter (\(_, pkgId) -> Just (pkgName pkgId) /= mbLibName) $ nub (ghcOptPackages ghcOpts')
                                        , ghcOptSourcePath = map (baseDir </>) (ghcOptSourcePath ghcOpts')
