@@ -9,8 +9,11 @@ module CommandLoop
 
 import Control.Monad (when)
 import Data.IORef
-import Data.List (find)
+import Data.List (find, intercalate)
+#if __GLASGOW_HASKELL__ < 709
+import Control.Applicative ((<$>))
 import Data.Traversable (traverse)
+#endif
 import MonadUtils (MonadIO, liftIO)
 import System.Directory (setCurrentDirectory)
 import System.Exit (ExitCode(ExitFailure, ExitSuccess))
@@ -25,7 +28,9 @@ import System.Posix.Files (getFileStatus, modificationTime)
 
 import Types (ClientDirective(..), Command(..), CommandExtra(..))
 import Info (getIdentifierInfo, getType)
+import FindSymbol (findSymbol)
 import Cabal (getPackageGhcOpts)
+import Stack
 
 type ClientSend = ClientDirective -> IO ()
 
@@ -54,17 +59,20 @@ mkCabalConfig path = do
 
 data Config = Config
     { configGhcOpts :: [String]
-    , configCabal :: Maybe CabalConfig
+    , configCabal   :: Maybe CabalConfig
+    , configStack   :: Maybe StackConfig
     }
     deriving Eq
 
 newConfig :: CommandExtra -> IO Config
 newConfig cmdExtra = do
     mbCabalConfig <- traverse mkCabalConfig $ ceCabalConfig cmdExtra
-    return $ Config { configGhcOpts = ceGhcOptions cmdExtra
-                    , configCabal = mbCabalConfig
-                    }
+    mbStackConfig <- getStackConfig cmdExtra
 
+    return $ Config { configGhcOpts = "-O0" : ceGhcOptions cmdExtra
+                    , configCabal = mbCabalConfig
+                    , configStack = mbStackConfig
+                    }
 
 type CommandObj = (Command, Config)
 
@@ -132,22 +140,21 @@ configSession state clientSend config = do
                           return $ Right []
                       Just cabalConfig -> do
                           liftIO $ setCurrentDirectory . takeDirectory $ cabalConfigPath cabalConfig
-                          liftIO $ getPackageGhcOpts $ cabalConfigPath cabalConfig
-
+                          liftIO $ getPackageGhcOpts (cabalConfigPath cabalConfig) (configStack config)
     case eCabalGhcOpts of
       Left e -> return $ Left e
       Right cabalGhcOpts -> do
-          let allGhcOpts = cabalGhcOpts ++ (configGhcOpts config)
-          GHC.gcatch (fmap Right $ updateDynFlags allGhcOpts)
+          let allGhcOpts = cabalGhcOpts ++ configGhcOpts config
+          GHC.gcatch (Right <$> updateDynFlags allGhcOpts)
                      (fmap Left . handleGhcError)
   where
     updateDynFlags :: [String] -> GHC.Ghc ()
     updateDynFlags ghcOpts = do
         initialDynFlags <- GHC.getSessionDynFlags
         let updatedDynFlags = initialDynFlags
-                { GHC.log_action = logAction state clientSend
-                , GHC.ghcLink = GHC.NoLink
-                , GHC.hscTarget = GHC.HscInterpreted
+                { GHC.log_action    = logAction state clientSend
+                , GHC.ghcLink       = GHC.NoLink
+                , GHC.hscTarget     = GHC.HscInterpreted
                 }
         (finalDynFlags, _, _) <- GHC.parseDynamicFlags updatedDynFlags (map GHC.noLoc ghcOpts)
         _ <- GHC.setSessionDynFlags finalDynFlags
@@ -156,7 +163,6 @@ configSession state clientSend config = do
     handleGhcError :: GHC.GhcException -> GHC.Ghc String
     handleGhcError e = return $ GHC.showGhcException e ""
 
-
 runCommand :: IORef State -> ClientSend -> Command -> GHC.Ghc ()
 runCommand _ clientSend (CmdCheck file) = do
     let noPhase = Nothing
@@ -225,6 +231,21 @@ runCommand state clientSend (CmdType file (line, col)) = do
             , show endCol , " "
             , "\"", t, "\""
             ]
+runCommand state clientSend (CmdFindSymbol symbol files) = do
+    result <- withWarnings state False $ findSymbol symbol files
+    case result of
+        []      -> liftIO $ mapM_ clientSend
+                       [ ClientStderr $ "Couldn't find modules containing '" ++ symbol ++ "'"
+                       , ClientExit (ExitFailure 1)
+                       ]
+        modules -> liftIO $ mapM_ clientSend
+                       [ ClientStdout (formatModules modules)
+                       , ClientExit ExitSuccess
+                       ]
+    where
+    formatModules = intercalate "\n"
+
+
 
 #if __GLASGOW_HASKELL__ >= 706
 logAction :: IORef State -> ClientSend -> GHC.DynFlags -> GHC.Severity -> GHC.SrcSpan -> Outputable.PprStyle -> ErrUtils.MsgDoc -> IO ()
