@@ -9,13 +9,15 @@ module Types
 import System.Exit (ExitCode)
 
 data CommandExtra = CommandExtra
-  { ceGhcOptions :: [String]
+  { ceGhcOptions  :: [String]
   , ceCabalConfig :: Maybe FilePath
+  , cePath        :: Maybe FilePath
   } deriving (Read, Show)
 
 emptyCommandExtra :: CommandExtra
-emptyCommandExtra = CommandExtra { ceGhcOptions = []
+emptyCommandExtra = CommandExtra { ceGhcOptions  = []
                                  , ceCabalConfig = Nothing
+                                 , cePath        = Nothing
                                  }
 
 data ServerDirective
@@ -36,4 +38,5 @@ data Command
     | CmdModuleFile String
     | CmdInfo FilePath String
     | CmdType FilePath (Int, Int)
+    | CmdFindSymbol String [String]
     deriving (Read, Show)
